//---------------------------------------------------------------------------


#pragma hdrstop

#include "Procs.h"
#include "MyConvertWAV.h"
#include "Main.h"
#include "Vox.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

bool GetFileFormat(LPTSTR FileName, LPWAVEFORMATEX wfx);

WAVE_FORMAT_INFO WAVformats[] = {
	{"Unknown" 										, 0x0000}, /* Microsoft Corporation */
	{"PCM"                      	, 0x0001}, /* Microsoft Corporation */
	{"Microsoft ADPCM"            , 0x0002}, /* Microsoft Corporation */
	{"IEEE_FLOAT"                 , 0x0003}, /* Microsoft Corporation */
	{"VSELP"                      , 0x0004}, /* Compaq Computer Corp. */
	{"IBM_CVSD"                   , 0x0005}, /* IBM Corporation */
	{"A-LAW"                      , 0x0006}, /* Microsoft Corporation */
	{"u-LAW"                     , 0x0007}, /* Microsoft Corporation */
	{"DTS"                       , 0x0008}, /* Microsoft Corporation */
	{"DRM"                       , 0x0009}, /* Microsoft Corporation */
	{"OKI_ADPCM"                 , 0x0010}, /* OKI */
	{"DVI ADPCM"                 , 0x0011}, /* Intel Corporation */
	{"MEDIASPACE_ADPCM"          , 0x0012}, /* Videologic */
	{"SIERRA_ADPCM"              , 0x0013}, /* Sierra Semiconductor Corp */
	{"G723_ADPCM"                , 0x0014}, /* Antex ElectronicS"Corporation */
	{"DIGISTD"                   , 0x0015}, /* DSP Solutions, Inc. */
	{"DIGIFIX"                   , 0x0016}, /* DSP Solutions, Inc. */
	{"Dialogic ADPCM"        			, 0x0017}, /* Dialogic Corporation */
	{"MEDIAVISION_ADPCM"         , 0x0018}, /* Media Vision, Inc. */
	{"CU_CODEC"                  , 0x0019}, /* Hewlett-PackarD"Company */
	{"YAMAHA_ADPCM"              , 0x0020}, /* Yamaha Corporation of America */
	{"SONARC"                    , 0x0021}, /* SpeecH"Compression */
	{"DSPGROUP_TRUESPEECH"       , 0x0022}, /* DSP Group, InC"*/
	{"ECHOSC1"                    , 0x0023}, /* Echo SpeecH"Corporation */
	{"AUDIOFILE_AF36"             , 0x0024}, /* Virtual Music, Inc. */
	{"APTX"                      , 0x0025}, /* Audio Processing Technology */
	{"AUDIOFILE_AF10"             , 0x0026}, /* Virtual Music, Inc. */
	{"PROSODY_1612"               , 0x0027}, /* Aculab plC"*/
	{"LRC"                       , 0x0028}, /* Merging TechnologieS"S.A. */
	{"DOLBY_AC2"                  , 0x0030}, /* Dolby LaboratorieS"*/
	{"GSM 6.10"                    , 0x0031}, /* Microsoft Corporation */
	{"MSNAUDIO"                   , 0x0032}, /* Microsoft Corporation */
	{"ANTEX_ADPCME"               , 0x0033}, /* AnteX"ElectronicS"Corporation */
	{"CONTROL_RES_VQLPC"         , 0x0034}, /* Control ResourceS"LimiteD"*/
	{"DIGIREAL"                   , 0x0035}, /* DSP Solutions, Inc. */
	{"DIGIADPCM"                 , 0x0036}, /* DSP Solutions, Inc. */
	{"CONTROL_RES_CR10"           , 0x0037}, /* Control ResourceS"LimiteD"*/
	{"NMS_VBXADPCM"              , 0x0038}, /* Natural MicroSystemS"*/
	{"CS_IMAADPCM"               , 0x0039}, /* Crystal Semiconductor IMA ADPCM"*/
	{"ECHOSC3"                    , 0x003A}, /* Echo SpeecH"Corporation */
	{"ROCKWELL_ADPCM"            , 0x003B}, /* Rockwell International */
	{"ROCKWELL_DIGITALK"          , 0x003C}, /* Rockwell International */
	{"XEBEC"                     , 0x003D}, /* XebeC"Multimedia SolutionS"LimiteD"*/
	{"G721_ADPCM"                , 0x0040}, /* AnteX"ElectronicS"Corporation */
	{"G728_CELP"                  , 0x0041}, /* AnteX"ElectronicS"Corporation */
	{"MSG723"                     , 0x0042}, /* Microsoft Corporation */
	{"MPEG"                       , 0x0050}, /* Microsoft Corporation */
	{"RT24"                       , 0x0052}, /* InSoft, Inc. */
	{"PAC"                       , 0x0053}, /* InSoft, Inc. */
	{"MPEGLAYER3"                 , 0x0055}, /* ISO/MPEG Layer3}Format Tag */
	{"LUCENT_G723"                , 0x0059}, /* Lucent TechnologieS"*/
	{"CIRRUS"                    , 0x0060}, /* CirruS"LogiC"*/
	{"ESPCM"                     , 0x0061}, /* ESS"Technology */
	{"VOXWARE"                    , 0x0062}, /* Voxware InC"*/
	{"CANOPUS_ATRAC"             , 0x0063}, /* Canopus, co., Ltd. */
	{"G726_ADPCM"                , 0x0064}, /* APICOM"*/
	{"G722_ADPCM"                , 0x0065}, /* APICOM"*/
	{"DSAT_DISPLAY"               , 0x0067}, /* Microsoft Corporation */
	{"VOXWARE_BYTE_ALIGNED"      , 0x0069}, /* Voxware InC"*/
	{"VOXWARE_AC8"                , 0x0070}, /* Voxware InC"*/
	{"VOXWARE_AC10"               , 0x0071}, /* Voxware InC"*/
	{"VOXWARE_AC16"               , 0x0072}, /* Voxware InC"*/
	{"VOXWARE_AC20"               , 0x0073}, /* Voxware InC"*/
	{"VOXWARE_RT24"               , 0x0074}, /* Voxware InC"*/
	{"VOXWARE_RT29"               , 0x0075}, /* Voxware InC"*/
	{"VOXWARE_RT29HW"            , 0x0076}, /* Voxware InC"*/
	{"VOXWARE_VR12"               , 0x0077}, /* Voxware InC"*/
	{"VOXWARE_VR18"               , 0x0078}, /* Voxware InC"*/
	{"VOXWARE_TQ40"               , 0x0079}, /* Voxware InC"*/
	{"SOFTSOUND"                 , 0x0080}, /* Softsound, Ltd. */
	{"VOXWARE_TQ60"               , 0x0081}, /* Voxware InC"*/
	{"MSRT24"                     , 0x0082}, /* Microsoft Corporation */
	{"G729A"                      , 0x0083}, /* AT&T Labs, Inc. */
	{"MVI_MVI2"                   , 0x0084}, /* Motion PixelS"*/
	{"DF_G726"                    , 0x0085}, /* DataFusion SystemS"(Pty) (Ltd) */
	{"DF_GSM610"                  , 0x0086}, /* DataFusion SystemS"(Pty) (Ltd) */
	{"ISIAUDIO"                   , 0x0088}, /* IterateD"Systems, Inc. */
	{"ONLIVE"                     , 0x0089}, /* OnLive! Technologies, Inc. */
	{"SBC24"                      , 0x0091}, /* SiemenS"BusinesS"CommunicationS"SyS"*/
	{"DOLBY_AC3_SPDIF"            , 0x0092}, /* SoniC"Foundry */
	{"MEDIASONIC_G723"            , 0x0093}, /* MediaSoniC"*/
	{"PROSODY_8KBPS"             , 0x0094}, /* Aculab plC"*/
	{"ZYXEL_ADPCM"               , 0x0097}, /* ZyXEL Communications, Inc. */
	{"PHILIPS_LPCBB"              , 0x0098}, /* PhilipS"SpeecH"Processing */
	{"PACKED"                    , 0x0099}, /* Studer Professional Audio AG */
	{"MALDEN_PHONYTALK"           , 0x00A0}, /* Malden ElectronicS"Ltd. */
	{"RHETOREX_ADPCM"            , 0x0100}, /* RhetoreX"Inc. */
	{"IRAT"                       , 0x0101}, /* BeCubeD"Software Inc. */
	{"VIVO_G723"                  , 0x0111}, /* Vivo Software */
	{"VIVO_SIREN"                 , 0x0112}, /* Vivo Software */
	{"DIGITAL_G723"               , 0x0123}, /* Digital Equipment Corporation */
	{"SANYO_LD_ADPCM"            , 0x0125}, /* Sanyo ElectriC"Co., Ltd. */
	{"SIPROLAB_ACEPLNET"          , 0x0130}, /* Sipro Lab TelecoM"Inc. */
	{"SIPROLAB_ACELP4800"         , 0x0131}, /* Sipro Lab TelecoM"Inc. */
	{"SIPROLAB_ACELP8V3"          , 0x0132}, /* Sipro Lab TelecoM"Inc. */
	{"SIPROLAB_G729"              , 0x0133}, /* Sipro Lab TelecoM"Inc. */
	{"SIPROLAB_G729A"             , 0x0134}, /* Sipro Lab TelecoM"Inc. */
	{"SIPROLAB_KELVIN"            , 0x0135}, /* Sipro Lab TelecoM"Inc. */
	{"G726ADPCM"                 , 0x0140}, /* Dictaphone Corporation */
	{"QUALCOMM_PUREVOICE"         , 0x0150}, /* Qualcomm, Inc. */
	{"QUALCOMM_HALFRATE"          , 0x0151}, /* Qualcomm, Inc. */
	{"TUBGSM"                    , 0x0155}, /* Ring Zero Systems, Inc. */
	{"MSAUDIO1"                   , 0x0160}, /* Microsoft Corporation */
	{"UNISYS_NAP_ADPCM"          , 0x0170}, /* UnisyS"Corp. */
	{"UNISYS_NAP_ULAW"           , 0x0171}, /* UnisyS"Corp. */
	{"UNISYS_NAP_ALAW"           , 0x0172}, /* UnisyS"Corp. */
	{"UNISYS_NAP_16K"             , 0x0173}, /* UnisyS"Corp. */
	{"CREATIVE_ADPCM"            , 0x0200}, /* Creative Labs, InC"*/
	{"CREATIVE_FASTSPEECH8"       , 0x0202}, /* Creative Labs, InC"*/
	{"CREATIVE_FASTSPEECH10"      , 0x0203}, /* Creative Labs, InC"*/
	{"UHER_ADPCM"                , 0x0210}, /* UHER informatiC"GmbH"*/
	{"QUARTERDECK"                , 0x0220}, /* Quarterdeck Corporation */
	{"ILINK_VC"                  , 0x0230}, /* I-link Worldwide */
	{"RAW_SPORT"                  , 0x0240}, /* Aureal Semiconductor */
	{"ESST_AC3"                   , 0x0241}, /* ESS"Technology, Inc. */
	{"IPI_HSX"                   , 0x0250}, /* Interactive Products, Inc. */
	{"IPI_RPELP"                  , 0x0251}, /* Interactive Products, Inc. */
	{"CS2"                        , 0x0260}, /* Consistent Software */
	{"SONY_SCX"                  , 0x0270}, /* Sony Corp. */
	{"FM_TOWNS_SND"              , 0x0300}, /* Fujitsu Corp. */
	{"BTV_DIGITAL"                , 0x0400}, /* Brooktree Corporation */
	{"QDESIGN_MUSIC"             , 0x0450}, /* QDesign Corporation */
	{"VME_VMPCM"                 , 0x0680}, /* AT&T Labs, Inc. */
	{"TPC"                       , 0x0681}, /* AT&T Labs, Inc. */
	{"OLIGSM"                    , 0x1000}, /* Ing C. Olivetti & C., S.p.A. */
	{"OLIADPCM"                  , 0x1001}, /* Ing C. Olivetti & C., S.p.A. */
	{"OLICELP"                    , 0x1002}, /* Ing C. Olivetti & C., S.p.A. */
	{"OLISBC"                    , 0x1003}, /* Ing C. Olivetti & C., S.p.A. */
	{"OLIOPR"                     , 0x1004}, /* Ing C. Olivetti & C., S.p.A. */
	{"LH_CODEC"                  , 0x1100}, /* Lernout & Hauspie */
	{"NORRIS"                    , 0x1400}, /* NorriS"Communications, Inc. */
	{"SOUNDSPACE_MUSICOMPRESS"   , 0x1500}, /* AT&T Labs, Inc. */
	{"DVM"                       , 0x2000},
	{NULL, 0xFFFF } /*   */
};

//---------------------------------------------------------------------------
BOOL ConvertFile(LPTSTR szSrcFile, LPTSTR szDestFile, ConvertInfo *ci)
{
    char TempFileName[MAX_PATH];
    bool IsSrcVox =
        ((MainForm->FileType->ItemIndex == 2) ||
        (ExtractFileExt(szSrcFile).LowerCase() == ".vox")) &&
        !(MainForm->FileType->ItemIndex == 1);
    bool Overwrite = !lstrcmpi(szSrcFile, szDestFile);
    AnsiString OldName;

    if (Overwrite)
    {
        OldName = szDestFile;
        GetTempPath(MAX_PATH, szDestFile);
        GetTempFileName(szDestFile, "wav", 0, szDestFile);
    }

    try
    {
    if (!IsSrcVox && !ci->ToVox) // WAV to WAV
        throw ConvertWAV(szSrcFile, szDestFile, &ci->Fmt,
            ci->Normalize, ci->UseFilter);
    else
    if (IsSrcVox && !ci->ToVox) // VOX to WAV
    {
        if (ci->Fmt.wFormatTag == WAVE_FORMAT_PCM &&
            ci->Fmt.nSamplesPerSec == ci->VoxFreq &&
            ci->Fmt.wBitsPerSample == 16 &&
            ci->Fmt.nChannels == 1)
            throw VoxToPCM(szSrcFile, szDestFile, ci->VoxFreq);
        else
        {
            GetTempPath(MAX_PATH, TempFileName);
            GetTempFileName(TempFileName, "wav", 0, TempFileName);
            if (!VoxToPCM(szSrcFile, TempFileName, ci->VoxFreq))
            {
                DeleteFile(TempFileName);
                throw FALSE;
            }
            if (!ConvertWAV(TempFileName, szDestFile, &ci->Fmt,
                ci->Normalize, ci->UseFilter))
            {
                DeleteFile(TempFileName);
                throw FALSE;
            }
            DeleteFile(TempFileName);
        }
    }
    else
    if (!IsSrcVox && ci->ToVox) // WAV to VOX
    {
        WAVEFORMATEX SrcFmt;
        if (!GetFileFormat(szSrcFile, &SrcFmt))
            throw FALSE;
        if (SrcFmt.wFormatTag == WAVE_FORMAT_PCM &&
            SrcFmt.nSamplesPerSec == ci->Fmt.nSamplesPerSec &&
            SrcFmt.wBitsPerSample == 16 &&
            SrcFmt.nChannels == 1)
            throw PCMToVox(szSrcFile, szDestFile);
        else
        {
            WAVEFORMATEX TempFmt;
            TempFmt.wFormatTag = WAVE_FORMAT_PCM;
            TempFmt.nSamplesPerSec = ci->VoxFreq;
            TempFmt.wBitsPerSample = 16;
            TempFmt.nChannels = 1;
            GetTempPath(MAX_PATH, TempFileName);
            GetTempFileName(TempFileName, "wav", 0, TempFileName);
            if (!ConvertWAV(szSrcFile, TempFileName, &TempFmt,
                ci->Normalize, ci->UseFilter))
            {
                DeleteFile(TempFileName);
                throw FALSE;
            }
            if (!PCMToVox(TempFileName, szDestFile))
            {
                DeleteFile(TempFileName);
                throw FALSE;
            }
            DeleteFile(TempFileName);
        }
    }
    else // VOX to VOX
    {
        WAVEFORMATEX TempFmt;
        char TempFileName2[MAX_PATH];
        if (ci->VoxFreq == ci->Fmt.nSamplesPerSec)
            throw CopyFile(szSrcFile, szDestFile, FALSE);
        GetTempPath(MAX_PATH, TempFileName);
        GetTempFileName(TempFileName, "wav", 0, TempFileName);
        if (!VoxToPCM(szSrcFile, TempFileName, ci->VoxFreq))
        {
            DeleteFile(TempFileName);
            throw FALSE;
        }
        TempFmt.wFormatTag = WAVE_FORMAT_PCM;
        TempFmt.nSamplesPerSec = ci->Fmt.nSamplesPerSec;
        TempFmt.wBitsPerSample = 16;
        TempFmt.nChannels = 1;
        GetTempPath(MAX_PATH, TempFileName2);
        GetTempFileName(TempFileName2, "wav", 0, TempFileName2);
        if (!ConvertWAV(TempFileName, TempFileName2, &TempFmt,
            ci->Normalize, ci->UseFilter))
        {
            DeleteFile(TempFileName);
            throw FALSE;
        }
        DeleteFile(TempFileName);
        if (!PCMToVox(TempFileName2, szDestFile))
        {
            DeleteFile(TempFileName2);
            throw FALSE;
        }
        DeleteFile(TempFileName2);
    }
    }
    catch (BOOL Res)
    {
        if (!Overwrite) return Res;
        if (!Res)
        {
            DeleteFile(szDestFile);
            return FALSE;
        }
        if (!DeleteFile(OldName))
            return FALSE;
        if (!RenameFile(szDestFile, OldName))
            return FALSE;
        return TRUE;
    }

    return true;
}
//---------------------------------------------------------------------------
BOOL ConvertWAV(LPTSTR szSrcFile, LPTSTR szDestFile,
    LPWAVEFORMATEX pDestFmt, BOOL Normalize, BOOL UseFilter)
{
	WAVEFORMATEX SrcFmt;

    pDestFmt->nBlockAlign = pDestFmt->wBitsPerSample * pDestFmt->nChannels / 8;
    pDestFmt->nAvgBytesPerSec = pDestFmt->nSamplesPerSec * pDestFmt->nBlockAlign;

    if (!GetFileFormat(szSrcFile, &SrcFmt))
        return FALSE;

    if (SrcFmt.wFormatTag == WAVE_FORMAT_PCM &&
        pDestFmt->wFormatTag == WAVE_FORMAT_PCM) // PCM to PCM
    {
        if (SrcFmt.nSamplesPerSec == pDestFmt->nSamplesPerSec &&
            SrcFmt.wBitsPerSample == pDestFmt->wBitsPerSample &&
            SrcFmt.nChannels == pDestFmt->nChannels)
            return CopyFile(szSrcFile, szDestFile, FALSE);
        else
            return MyConvertWAV(szSrcFile, szDestFile,
                pDestFmt, Normalize, UseFilter);
    }
    else
    if (SrcFmt.wFormatTag != WAVE_FORMAT_PCM &&
        pDestFmt->wFormatTag == WAVE_FORMAT_PCM) // Non PCM to PCM
    {
        AnsiString TempFile;
        WAVEFORMATEX TempFmt = SrcFmt;
        TempFmt.cbSize = 0;
        TempFmt.wFormatTag = WAVE_FORMAT_PCM;
        if (!TempFmt.wBitsPerSample)
            TempFmt.wBitsPerSample = pDestFmt->wBitsPerSample;
        TempFmt.nBlockAlign = TempFmt.wBitsPerSample * TempFmt.nChannels / 8;
        TempFmt.nAvgBytesPerSec = TempFmt.nSamplesPerSec * TempFmt.nBlockAlign;

        if (ConvertToPCM(szSrcFile, HWND_DESKTOP, TempFile, &TempFmt) & 0x80000000)
        {
            DeleteFile(TempFile);
            return FALSE;
        }
        if (!MyConvertWAV(TempFile.c_str(), szDestFile,
            pDestFmt, Normalize, UseFilter))
        {
            DeleteFile(TempFile);
            DeleteFile(szDestFile);
            return FALSE;
        }
        DeleteFile(TempFile);
    }
    else return FALSE; // ? -> Non PCM not supported

    return TRUE;
}
//---------------------------------------------------------------------------
HRESULT ConvertToPCM(const AnsiString &szSrcFile, HWND hwnd, AnsiString &szDestFile, WAVEFORMATEX *pwfx) {
	WAVEFORMATEX *pwfxIn;
	WAVEFORMATEX wfxOut;
	DWORD dwRead;
	char *pszBuf;
	RIFF_HEADER *pHeader;
	void *pInBuf;
	void *pOutBuf;
	DWORD dwOutBufSize;
	DWORD dwInBufSize;
	HACMSTREAM hACMS;
	MMRESULT mmr;
	ACMSTREAMHEADER ACMheader;
	ACMFORMATCHOOSE ACMChoose;
	bool bFreeInBuffer = false;

	if (pwfx) {
		wfxOut.wFormatTag = pwfx->wFormatTag;
		wfxOut.nChannels = pwfx->nChannels;
		wfxOut.nSamplesPerSec = pwfx->nSamplesPerSec;
		wfxOut.wBitsPerSample = pwfx->wBitsPerSample;
		wfxOut.nBlockAlign = pwfx->wBitsPerSample * pwfx->nChannels / 8;
		wfxOut.nAvgBytesPerSec = wfxOut.nBlockAlign * wfxOut.nSamplesPerSec;
		wfxOut.cbSize = 0;

	} else {

		wfxOut.wFormatTag = 1;
		wfxOut.nChannels = 1;
		wfxOut.nSamplesPerSec = 22050;
		wfxOut.nAvgBytesPerSec = 22050;
		wfxOut.nBlockAlign = 1;
		wfxOut.wBitsPerSample = 8;
		wfxOut.cbSize = 0;

		ACMChoose.cbStruct = sizeof(ACMChoose);
		ACMChoose.fdwStyle = ACMFORMATCHOOSE_STYLEF_INITTOWFXSTRUCT;
		ACMChoose.hwndOwner = hwnd;
		ACMChoose.pwfx = &wfxOut;
		ACMChoose.cbwfx = sizeof(wfxOut);
		ACMChoose.pszTitle = NULL;
		ACMChoose.szFormatTag[0] = 0;
		ACMChoose.szFormat[0] = 0;
		ACMChoose.pszName = NULL;
		ACMChoose.cchName = 0;
		ACMChoose.fdwEnum = 0;
		ACMChoose.pwfxEnum = NULL;
		ACMChoose.hInstance = NULL;
		ACMChoose.pszTemplateName = NULL;
		ACMChoose.lCustData = 0;
		ACMChoose.pfnHook = NULL;

		mmr = acmFormatChoose(&ACMChoose);
		if (mmr != MMSYSERR_NOERROR) return mmr;
	}

	HANDLE hFile = CreateFile(szSrcFile.c_str(),
														GENERIC_READ,
														FILE_SHARE_READ,
														NULL,
														OPEN_EXISTING,
														FILE_ATTRIBUTE_NORMAL,
														NULL);
	if (hFile == INVALID_HANDLE_VALUE) return -1;

	DWORD dwFileSize = GetFileSize(hFile, NULL);
	pszBuf = new char[dwFileSize];

	try {
		ReadFile(hFile, pszBuf, dwFileSize, &dwRead, NULL);
		CloseHandle(hFile);

		pHeader = (RIFF_HEADER *)pszBuf;
		if (pHeader->dwID != MAKEFOURCC('R', 'I', 'F', 'F') &&
				*(DWORD *)(pHeader + 1) != MAKEFOURCC('W', 'A', 'V', 'E'))
                    return -1;

		dwRead = 12;
		(DWORD)pHeader += 12;

		while (dwRead < dwFileSize) {
			if (pHeader->dwID == MAKEFOURCC('f', 'm', 't', ' ')) {
				pwfxIn = (WAVEFORMATEX *)(pHeader + 1);
//				memcpy(pwfxIn, pHeader + 1, pHeader->dwSize);
//				wfxIn.cbSize = 0;
			}
			if (pHeader->dwID == MAKEFOURCC('d', 'a', 't', 'a')) {
				dwInBufSize = pHeader->dwSize;
				pInBuf = pHeader + 1;
				dwRead += sizeof(RIFF_HEADER) + pHeader->dwSize;
				(DWORD)pHeader += sizeof(RIFF_HEADER) + pHeader->dwSize;
				break;
			}
			dwRead += sizeof(RIFF_HEADER) + pHeader->dwSize;
			(DWORD)pHeader += sizeof(RIFF_HEADER) + pHeader->dwSize;
		}

		WAVEFORMATEX wfxTest = wfxOut;

		mmr = acmFormatSuggest(
			NULL,
			pwfxIn,
			&wfxTest,
			sizeof(wfxTest),
			ACM_FORMATSUGGESTF_NCHANNELS |
			ACM_FORMATSUGGESTF_NSAMPLESPERSEC |
			ACM_FORMATSUGGESTF_WBITSPERSAMPLE |
			ACM_FORMATSUGGESTF_WFORMATTAG);

		switch (mmr) {
			case MMSYSERR_NOERROR:
                pOutBuf = pInBuf;
                dwOutBufSize = dwInBufSize;
                *pwfx = *pwfxIn;
                wfxOut = *pwfxIn;
                break;

			case ACMERR_NOTPOSSIBLE: {
				mmr = acmFormatSuggest(
					NULL,
					pwfxIn,
					&wfxTest,
					sizeof(wfxTest),
//					ACM_FORMATSUGGESTF_NCHANNELS |
//					ACM_FORMATSUGGESTF_NSAMPLESPERSEC |
//					ACM_FORMATSUGGESTF_WBITSPERSAMPLE |
					ACM_FORMATSUGGESTF_WFORMATTAG);

				if (mmr) return mmr;

				mmr = acmStreamOpen(&hACMS,
												NULL,
												pwfxIn,
												&wfxTest,
												NULL,
												0,
												0,
												ACM_STREAMOPENF_NONREALTIME);
				if (mmr) return mmr;

                *pwfx = wfxTest;

				mmr = acmStreamSize(hACMS, dwInBufSize, &dwOutBufSize, ACM_STREAMSIZEF_SOURCE);
				if (mmr) return mmr;

				pOutBuf = new char[dwOutBufSize];

				ACMheader.cbStruct = sizeof(ACMheader);
				ACMheader.fdwStatus = 0;
				ACMheader.dwUser = 0;
				ACMheader.pbSrc = (LPBYTE)pInBuf;
				ACMheader.cbSrcLength = dwInBufSize;
				ACMheader.cbSrcLengthUsed = 0;
				ACMheader.dwSrcUser = 0;
				ACMheader.pbDst = (LPBYTE)pOutBuf;
				ACMheader.cbDstLength = dwOutBufSize;
				ACMheader.cbDstLengthUsed = 0;
				ACMheader.dwDstUser = 0;

				mmr = acmStreamPrepareHeader(hACMS, &ACMheader, 0);
				if (mmr) return mmr;

				mmr = acmStreamConvert(hACMS, &ACMheader, ACM_STREAMCONVERTF_BLOCKALIGN);
				if (mmr) return mmr;

				mmr = acmStreamReset(hACMS, 0);
				if (mmr) return mmr;

				mmr = acmStreamUnprepareHeader(hACMS, &ACMheader, 0);
				if (mmr) return mmr;

				mmr = acmStreamClose(hACMS, 0);
				if (mmr) return mmr;

				*pwfxIn = wfxTest;
                wfxOut = wfxTest;

				pInBuf = pOutBuf;
				dwInBufSize = dwOutBufSize;
				bFreeInBuffer = true;

				break;
			}
			default:
				return mmr;
		}


		try {
			DWORD dwHeaderSize = sizeof(RIFF_HEADER) + sizeof(DWORD) + sizeof(RIFF_HEADER) + 16 + sizeof(RIFF_HEADER);
			DWORD dwOutFileSize = dwHeaderSize + dwOutBufSize;
			if (*(pszBuf + dwRead) == 'L') dwOutFileSize += dwFileSize - dwRead;

			char *pszOutFile = new char[dwOutFileSize];

			// 
			*(DWORD *)pszOutFile = MAKEFOURCC('R', 'I', 'F', 'F');
			*(DWORD *)(pszOutFile + 4) = dwOutFileSize - sizeof(RIFF_HEADER);
			*(DWORD *)(pszOutFile + 8) = MAKEFOURCC('W', 'A', 'V', 'E');
			*(DWORD *)(pszOutFile + 12) = MAKEFOURCC('f', 'm', 't', ' ');
			*(DWORD *)(pszOutFile + 16) = 16;
			memcpy(pszOutFile + 20, &wfxOut, 16);

			//  
			*(DWORD *)(pszOutFile + 36) = MAKEFOURCC('d', 'a', 't', 'a');
			*(DWORD *)(pszOutFile + 40) = dwOutBufSize;
			memcpy(pszOutFile + dwHeaderSize, pOutBuf, dwOutBufSize);

			// LIST 
			if (*(pszBuf + dwRead) == 'L')
				memcpy(pszOutFile + dwHeaderSize + dwOutBufSize, pszBuf + dwRead, dwFileSize - dwRead);

			char szINIPath[MAX_PATH];
			char szINI[MAX_PATH];

			GetTempPath(sizeof(szINIPath), szINIPath);
			GetTempFileName(szINIPath,
											"wav",
											0,
											szINI);

			szDestFile = szINI;
			int nFile = FileCreate(szDestFile);
			FileWrite(nFile, pszOutFile, dwOutFileSize);
			FileClose(nFile);

			delete[] pszOutFile;
		} __finally {

			if (bFreeInBuffer) delete[] pInBuf;
			delete[] (char *)pOutBuf;
		}
	} __finally {
		delete[] pszBuf;
	}

	return 0;
}
//---------------------------------------------------------------------------
AnsiString SizeToStr(DWORD Size)
{
    AnsiString t;
    if (Size < 0x400)
    {
        t.printf("%d ", Size);
        if ((Size % 100) / 10 == 1 ||
            (Size % 10) < 2 ||
            (Size % 10) > 4)
            t += " ";
        else
            t += " ";
    }
    else if (Size < 0x100000)
        t.printf("%.2f ", (double)Size / 0x400);
    else if (Size < 0x40000000)
        t.printf("%.2f ", (double)Size / 0x100000);
    else
        t.printf("%.2f ", (double)Size / 0x40000000);

    return t;
}
//---------------------------------------------------------------------------
bool GetFileFormat(LPTSTR FileName, LPWAVEFORMATEX wfx)
{
	DWORD Read;
	WAVHeader whdr;
	WAVSection wsec;
    HANDLE hSrcFile;

    try
    {
        if ((hSrcFile = CreateFile(FileName, GENERIC_READ, FILE_SHARE_READ,
            NULL, OPEN_EXISTING, 0, 0)) == INVALID_HANDLE_VALUE)
            throw 0;
        DWORD FileSize = GetFileSize(hSrcFile, NULL);
        ReadFile(hSrcFile, &whdr, sizeof(whdr), &Read, NULL);
        if (memcmp(whdr.RIFF.Name, "RIFF", 4)) throw 0;
        if (FileSize - whdr.RIFF.Size != 8) throw 0;
        if (memcmp(whdr.WAVE, "WAVE", 4)) throw 0;
        if (memcmp(whdr.fmt.Name, "fmt ", 4)) throw 0;
        ReadFile(hSrcFile, wfx, 0x10, &Read, NULL);
    }
    catch (int)
    {
        CloseHandle(hSrcFile);
        return false;
    }
    CloseHandle(hSrcFile);
    return true;
}
//---------------------------------------------------------------------------
AnsiString FormatToStr(LPTSTR FileName)
{
    WAVEFORMATEX wfx;
    if (ExtractFileExt(FileName).LowerCase() == ".vox")
        return "Dialogic ADPCM";
    if (!GetFileFormat(FileName, &wfx))
        return "  ";
    int i = 0, k = sizeof(WAVformats) / sizeof(WAVE_FORMAT_INFO);
    AnsiString FormatTag = "";
    for (; i < k; i++)
        if (wfx.wFormatTag == WAVformats[i].wFormat)
        {
            FormatTag = WAVformats[i].pszName;
            break;
        }
    return FormatTag + ' ' + (wfx.nSamplesPerSec / 1000) + "  " +
        wfx.wBitsPerSample + "  " + (wfx.nChannels == 1 ? "" : "");
}
//---------------------------------------------------------------------------

